;;;
;;; scmail - a mail filter written in Scheme
;;;
;;; Copyright (C) 2002-2004 Satoru Takabayashi <satoru@namazu.org> 
;;;     All rights reserved.
;;;     This is free software with ABSOLUTELY NO WARRANTY.
;;;
;;; Permission to use, copy, modify, distribute this software and
;;; accompanying documentation for any purpose is hereby granted,
;;; provided that existing copyright notices are retained in all
;;; copies and that this notice is included verbatim in all
;;; distributions.
;;; This software is provided as is, without express or implied
;;; warranty.  In no circumstances the author(s) shall be liable
;;; for any damages arising out of the use of this software.
;;;

(define-module scmail.config-core
  (use file.util)
  (use gauche.parameter)
  (export <config>
          scmail-config
          scmail-config-set-verbose-mode!
          scmail-config-verbose-mode?
	  ))
(select-module scmail.config-core)

(define-class <config> ()
  ((mailbox :init-value "~/Mail"
	    :init-keyword :mailbox)
   (mailbox-type :init-value 'mh
                 :init-keyword :mailbox-type)
   (inbox :init-value "inbox"
	  :init-keyword :inbox)
   (spam  :init-value "spam"
	  :init-keyword :spam)
   (umask :init-value #o077
          :init-keyword :umask)
   (size-limit :init-value (* (* 1024 1024) 10) ; 10 MB
               :init-keyword :size-limit)
   (smtp-host :init-value "localhost"
	      :init-keyword :smtp-host)
   (log-file :init-value "log"
	     :init-keyword :log-file)
   (deliver-rules :init-value "deliver-rules"
                  :init-keyword :deliver-rules)
   (refile-rules :init-value "refile-rules"
                 :init-keyword :refile-rules)
   (token-table :init-value "token-table.dbm"
                :init-keyword :token-table)
   (digest :init-value "digest.dbm"
            :init-keyword :digest)
   (verbose-mode :init-value #f)

   ;; for backward compatibility
   (junk  :init-value #f
	  :init-keyword :junk)
   ))

(define-method initialize ((config <config>) initargs)
  (next-method)
  (if (not (or (eq? (ref config 'mailbox-type) 'mh)
               (eq? (ref config 'mailbox-type) 'maildir)))
      (errorf "unsupported mailbox-type: ~a" (ref config 'mailbox-type)))
  (if (ref config 'junk)
      (slot-set! config 'spam (ref config 'junk)))
  
  (for-each (lambda (slot)
              (slot-set! config slot (expand-path (ref config slot))))
            (list 'log-file 'refile-rules 'deliver-rules 'mailbox
                  'token-table 'digest))
)

(define-method write-object ((config <config>) port)
  (format port "[config]"))

(define scmail-config (make-parameter (make <config>)))

(define (scmail-config-set-verbose-mode!) 
  (slot-set! (scmail-config) 'verbose-mode #t))

(define (scmail-config-verbose-mode?)
  (ref (scmail-config) 'verbose-mode))

(provide "scmail/config-core")
