;;;
;;; scmail - a mail filter written in Scheme
;;;
;;; Copyright (C) 2002-2004 Satoru Takabayashi <satoru@namazu.org> 
;;;     All rights reserved.
;;;     This is free software with ABSOLUTELY NO WARRANTY.
;;;
;;; Permission to use, copy, modify, distribute this software and
;;; accompanying documentation for any purpose is hereby granted,
;;; provided that existing copyright notices are retained in all
;;; copies and that this notice is included verbatim in all
;;; distributions.
;;; This software is provided as is, without express or implied
;;; warranty.  In no circumstances the author(s) shall be liable
;;; for any damages arising out of the use of this software.
;;;

(define-module scmail.config
  (use file.util)
  (use gauche.parameter)
  (use scmail.config-core)
  (use scmail.util)
  (export <config>
          scmail-config-set-directory!
          scmail-config-default-file
          scmail-config-get-path
          scmail-config-make-directory
	  scmail-config-read
          scmail-config
          scmail-config-set-verbose-mode!
          scmail-config-verbose-mode?
	  ))
(select-module scmail.config)

(define scmail-config-directory (make-parameter (expand-path "~/.scmail")))

(define (scmail-config-set-directory! directory)
  (scmail-config-directory directory))

(define (build-config-path path)
  (if (absolute-path? path)
      path
      (build-path (scmail-config-directory) path)))

(define (scmail-config-read config-file)
  (with-error-handler
   (lambda (e) (make <config>))
   (lambda ()
     (let ((options (call-with-input-file config-file
		      (lambda (port) (read port)))))
       (scmail-config (apply make <config> options))
       (scmail-config)))))

;; Return an old path if it is existed for backward compatibility.
(define (choose path)
  (define table
    (map (lambda (pair) (cons (build-config-path (car pair))
                              (expand-path (cdr pair))))
         '(("config"          . "~/.scmailrc")
           ("refile-rules"    . "~/.scmailrc-refile")
           ("deliver-rules"   . "~/.scmailrc-deliver"))))
  (let* ((new-path (expand-path path))
         (old-path (let1 pair (assoc new-path table)
                        (if pair (expand-path (cdr pair)) #f))))
    (if (and old-path
             (not (file-exists? new-path))
             (file-exists? old-path))
        old-path
        new-path)))
  

(define (scmail-config-default-file)
  (choose (build-config-path  "config")))

(define (scmail-config-get-path slot)
  (choose (build-config-path (ref (scmail-config) slot))))

(define (scmail-config-make-directory)
  (let1 path (scmail-config-directory)
        (unless (file-exists? path)
                (with-error-handler
                 (lambda (e)
                   (scmail-eformat "~a" (ref e 'message)))
                 (lambda ()
                   (create-directory* path))))
        (if (file-exists? path)
            (begin 
              (unless (file-is-directory? path)
                      (scmail-eformat "~a is not a directory"))
              (unless (file-is-writable? path)
                      (scmail-eformat "~a is not writable"))))
        ))

(provide "scmail/config")
